# -*- coding: utf-8 -*-
import calendar

def split_seconds_to_clock(seconds):
    """Convert a time in seconds to a string like '15:22:01'"""
    m, s = divmod(seconds, 60)
    h, m = divmod(m, 60)
    return {
        'hour': h,
        'minute': m,
        'second': s
    }

def format_seconds_to_clock(seconds):
    """Convert a time in seconds to a string like '15:22:01'"""
    #return "%d:%02d:%02d" % split_second_to_clock(seconds)
    return "{hour}:{minute:02}".format(**split_seconds_to_clock(seconds))

def time_to_seconds(timeobj):
    """Convert a time to seconds"""
    return (timeobj.hour*60*60)+(timeobj.minute*60)+timeobj.second

def timedelta_to_seconds(delta):
    """Convert a timedelta to seconds, only taking care of days and seconds (that is left unchanged)"""
    return (delta.days*24*60*60)+delta.seconds

def week_from_date(dt, start=1):
    """
    Return the week number of the month from a datetime.Date, this is index on 1 by 
    default, use 'start' argument to change this
    """
    for week_no, week in enumerate(calendar.Calendar().monthdayscalendar(dt.year, dt.month), start=start):
        if dt.day in week:
            return week_no
    # This should never happend
    raise ValueError



"""
Crispy forms layouts
"""
from django import forms
from django.utils.translation import ugettext as _

from crispy_forms.helper import FormHelper
from crispy_forms_foundation.layout import Layout, Fieldset, SplitDateTimeField, Row, Div, Column, HTML, Field, InlineField, SwitchField, ButtonHolder, ButtonHolderPanel, Submit

def day_helper(form_tag=True, form_action='.', has_next=False):
    """
    DayEntry's form layout helper
    """
    helper = FormHelper()
    helper.form_action = form_action
    helper.attrs = {'data_abide': ''}
    helper.form_tag = form_tag
    
    buttons = [
        HTML('<ul class="button-group stack-for-small right"><li>'),
        Submit('submit', _('Save')),
        HTML('</li></ul>'),
    ]
    if has_next:
        buttons = [buttons[0]]+[
            HTML('</li><li>'),
            Submit('submit_and_next', _('Save and continue to next day')),
        ]+buttons[1:]
    
    helper.layout = Layout(
        Row(
            Column('vacation', css_class='small-6 medium-4 medium-offset-8 text-right'),
        ),
        Row(
            Column('start_datetime', css_class='small-6 medium-4'),
            Column('pause', css_class='small-6 medium-2'),
            Column('stop_datetime', css_class='small-6 medium-4'),
            Column('overtime', css_class='small-6 medium-2'),
            css_class='opacited'
        ),
        Row(
            Column('content', css_class='small-12'),
            css_class='opacited'
        ),
        ButtonHolderPanel(
            *buttons,
            css_class=' clearfix text-right'
        ),
    )

    return helper


def month_helper(form_tag=True):
    """
    Datebook's form layout helper
    """
    helper = FormHelper()
    helper.form_action = '.'
    helper.attrs = {'data_abide': ''}
    helper.form_tag = form_tag
    
    helper.layout = Layout(
        Row(
            Column(
                Field('owner'),
                css_class='small-12 medium-7 hide-label'
            ),
            Column(
                Field('period'),
                css_class='small-12 medium-3 hide-label'
            ),
            Column(
                ButtonHolder(
                    Submit(
                        'submit',
                        _('Save'),
                        css_class='tiny expand',
                    ),
                    css_class='text-right',
                ),
                css_class='small-12 medium-2'
            ),
            css_class='collapse',
        ),
    )
    
    return helper

def year_helper(form_tag=True):
    """
    Datebook's year form layout helper
    """
    helper = FormHelper()
    helper.form_action = '.'
    helper.form_class = "hide-label right clearfix"
    helper.attrs = {'data_abide': ''}
    helper.form_tag = form_tag
    
    helper.layout = Layout(
        Field(
            'year',
            wrapper_class='left',
        ),
        Submit(
            'submit',
            _('New year'),
            css_class='tiny',
        ),
    )
    
    return helper



# -*- coding: utf-8 -*-
"""
Datebook month views
"""
import datetime

from django import http
from django.views import generic
from django.core.urlresolvers import reverse

from braces.views import LoginRequiredMixin, PermissionRequiredMixin

from datebook.forms.month import DatebookForm
from datebook.models import Datebook
from datebook.mixins import DatebookCalendarMixin, OwnerOrPermissionRequiredMixin
from datebook.utils import format_seconds_to_clock

class DatebookMonthFormView(PermissionRequiredMixin, generic.FormView):
    """
    Datebook create form view
    """
    model = Datebook
    form_class = DatebookForm
    template_name = 'datebook/month/form.html'
    permission_required = 'datebook.add_datebook'
    raise_exception = True
    
    def form_valid(self, form):
        self.object = form.save()
        return super(DatebookMonthFormView, self).form_valid(form)
    
    def get_success_url(self):
        return self.object.get_absolute_url()


class DatebookMonthAddView(DatebookCalendarMixin, OwnerOrPermissionRequiredMixin, generic.View):
    """
    Automatically create the datebook if it does not allready exists for the "author+year+month" 
    kwargs given, then redirect to its page.
    
    If the Datebook allready exists for the given kwargs, raise a "Http404"
    """
    permission_required = 'datebook.add_datebook'
    raise_exception = True
    
    def get(self, request, *args, **kwargs):
        if Datebook.objects.filter(author=self.author, period__year=self.year, period__month=self.month).count()>0:
            raise http.Http404
        
        d = self.author.datebook_set.create(period=datetime.date(year=self.year, month=self.month, day=1))
        
        return http.HttpResponseRedirect(d.get_absolute_url())


class DatebookMonthView(LoginRequiredMixin, DatebookCalendarMixin, generic.TemplateView):
    """
    Datebook month details view
    
    Get the Calendar for the given year+month then fill it with day entries
    """
    template_name = "datebook/month/calendar.html"
    
    def get_calendar(self, day_filters={}):
        # Add current day if the datebook period is the current month+year
        #current_day = None
        current_day = datetime.date.today()
        #print _curr.replace(day=1), self.object.period
        #if _curr.replace(day=1) == self.object.period:
            #current_day = _curr
        
        _cal = super(DatebookMonthView, self).get_calendar()
        
        # Calculate total elapsed time for worked days
        day_entries = self.get_dayentry_list(day_filters)
        total_elapsed_time = total_overtime_seconds = 0
        for item in day_entries:
            if item.vacation:
                continue
            total_elapsed_time += item.get_elapsed_seconds()
            total_overtime_seconds += item.get_overtime_seconds()
        
        return {
            "weekheader": _cal.formatweekheader(),
            "month": _cal.formatmonth(self.object.period.year, self.object.period.month, dayentries=day_entries, current_day=current_day),
            "total_elapsed_time": format_seconds_to_clock(total_elapsed_time),
            "total_overtime_seconds": total_overtime_seconds,
            "total_overtime_time": format_seconds_to_clock(total_overtime_seconds),
        }
        
    def get_context_data(self, **kwargs):
        context = super(DatebookMonthView, self).get_context_data(**kwargs)
        context.update({
            'datebook': self.object,
            'datebook_calendar': self.get_calendar(),
        })
        return context
    
    def get(self, request, *args, **kwargs):
        self.object = self.get_datebook({'period__year': self.year, 'period__month': self.month})
        
        context = self.get_context_data(**kwargs)
        
        return self.render_to_response(context)



"""
Here we try to safely use crispy_form if installed
"""
from django.utils.translation import ugettext as _

from datebook.utils.imports import safe_import_module

# Try to import "crispy-forms" base stuff to use for the default helper
try:
    from crispy_forms.helper import FormHelper
    from crispy_forms.layout import Submit
except ImportError:
    # Dummy objects when crispy-forms is not available
    def default_helper():
        return None
    class CrispyFormMixin(object): pass
else:
    # Ok, crispy-forms is installed
    def default_helper(form_tag=True):
        helper = FormHelper()
        helper.form_action = '.'
        helper.form_tag = form_tag
        helper.add_input(Submit('submit', _('Submit')))
        return helper

    class CrispyFormMixin(object):
        """
        Embed the technic in a form mixin to use "crispy-forms" and safely fallback if not installed
        
        Mixin attributes that you can define to change behavior :
        
        * crispy_form_helper_path: Python path to the helper;
        * crispy_form_helper_kwargs: Kwargs dict to give to the helper when initialized;
        * crispy_form_tag: A boolean, add <form> tag if True;
        """
        crispy_form_helper_path = None # Custom layout method path
        crispy_form_helper_kwargs = {}
        crispy_form_tag = True
        
        def __init__(self, *args, **kwargs):
            # Specified helper if any (and import succeed)
            helper = safe_import_module(self.crispy_form_helper_path, default=default_helper)
            if helper is not None:
                self.helper = helper(form_tag=self.crispy_form_tag, **self.crispy_form_helper_kwargs)
            else:
                # Default helper
                self.helper = default_helper(form_tag=self.crispy_form_tag)



# -*- coding: utf-8 -*-
"""
Datebook week views

DEPRECATED
"""
import calendar

from django import http
from django.views import generic

from braces.views import LoginRequiredMixin

from datebook.mixins import DatebookCalendarMixin

class DatebookWeekView(LoginRequiredMixin, DatebookCalendarMixin, generic.TemplateView):
    """
    Datebook month week details view
    
    Get the week days from the calendar for the given year+month then merge it with 
    associated day entries
    """
    template_name = "datebook/week.html"
    
    def find_weekday_range(self):
        # week number is indexed on 1 in urls, but we use it at a position item in 
        # returned list, so we turn it to indexed on 0
        week = self.week-1
        
        try:
            weekdays = calendar.Calendar().monthdatescalendar(self.year, self.month)[week]
        except IndexError:
            raise http.Http404
        
        # Calendar fill the grid with previous or past month days, so we filter them to 
        # return only the current month days
        _f = lambda x: x.year == self.year and x.month == self.month
        return filter(_f, weekdays)
    
    def get_dayentry_list(self, *args, **kwargs):
        """Improve the queryset to get only the days for the given week"""
        if len(self.weekdays)>1:
            _f = {'activity_date__gte': self.weekdays[0], 'activity_date__lte': self.weekdays[-1]}
        else:
            _f = {'activity_date': self.weekdays[0]}
        return super(DatebookWeekView, self).get_dayentry_list(_f)
    
    def get_context_data(self, **kwargs):
        context = super(DatebookWeekView, self).get_context_data(**kwargs)
        
        # Filter query set on the week days
        self.weekdays = self.find_weekday_range()
        dayentry_list = self.get_dayentry_list()
        # Make an empty dict (index on date) of weekdays
        _w = dict(map(lambda x: (x, None), self.weekdays))
        # Make a dict from the day entry and merge it in the weekdays
        _w.update(dict(map(lambda x: (x.activity_date, x), dayentry_list)))
        
        context.update({
            'datebook': self.object,
            'week': self.week,
            # For django templates loop we need to have a sorted list
            'weekdays': [(key, _w[key]) for key in sorted(_w.iterkeys())],
        })
        return context
    
    def get(self, request, *args, **kwargs):
        # Week number can't be empty or zero (index on 1 for humans)
        if not self.week:
            raise http.Http404
        
        self.object = self.get_datebook({'period__year': self.year, 'period__month': self.month})
        
        context = self.get_context_data(**kwargs)
        
        return self.render_to_response(context)



# -*- coding: utf-8 -*-
"""
View mixins
"""
import datetime

from django.shortcuts import get_object_or_404
from django.contrib.auth.models import User
from django.core.exceptions import ImproperlyConfigured, PermissionDenied
from django.contrib.auth.views import redirect_to_login

from braces.views import PermissionRequiredMixin

from datebook.models import Datebook
from datebook.calendars import DatebookCalendar

class AuthorKwargsMixin(object):
    """
    Get the User object for the given "author" kwarg if given
    """
    set_as_attr = True # IF False, don't set the object as object attribute in dispatch
    key_name = 'author'
    
    def dispatch(self, request, *args, **kwargs):
        if self.set_as_attr and self.key_name in kwargs:
            setattr(self, self.key_name, self.get_author(kwargs[self.key_name]))
        return super(AuthorKwargsMixin, self).dispatch(request, *args, **kwargs)
    
    def get_author(self, name):
        return get_object_or_404(User, username=name)
        
    def get_context_data(self, **kwargs):
        context = super(AuthorKwargsMixin, self).get_context_data(**kwargs)
        if hasattr(self, self.key_name):
            context[self.key_name] = getattr(self, self.key_name)
        return context


class DateKwargsMixin(AuthorKwargsMixin):
    """
    Get common date kwargs (year, month, day) to set them as object attributes
    
    This does not check anything, kwargs rules have to be in your view or urls map
    """
    date_kwarg_names = ['year', 'month', 'week', 'day']
    
    def dispatch(self, request, *args, **kwargs):
        for name in self.date_kwarg_names:
            if name in kwargs:
                setattr(self, name, int(kwargs[name]))
        return super(DateKwargsMixin, self).dispatch(request, *args, **kwargs)
        
    def get_context_data(self, **kwargs):
        context = super(DateKwargsMixin, self).get_context_data(**kwargs)
        for name in self.date_kwarg_names:
            context[name] = getattr(self, name, None)
        context['target_date'] = datetime.date(getattr(self, 'year', 1977), getattr(self, 'month', 1), getattr(self, 'day', 1))
        context['today'] = datetime.datetime.today()
        return context


class DatebookCalendarMixin(DateKwargsMixin):
    """
    Datebook calendar mixin
    """
    calendar_obj = DatebookCalendar
    
    def get_datebook(self, filters):
        return get_object_or_404(Datebook, author__username=self.kwargs['author'], **filters)
    
    def get_dayentry_list(self, filters={}):
        return self.object.dayentry_set.filter(**filters).order_by('activity_date')
    
    def get_calendar(self, *args, **kwargs):
        return self.calendar_obj(*args, **kwargs)


class DatebookCalendarAutoCreateMixin(DatebookCalendarMixin):
    """
    Same as DatebookCalendarMixin but use a get_or_create instead of get_object_or_404 
    so the datebook is automatically created
    """
    def get_current_date(self):
        today = datetime.date.today()
        self.year = self.kwargs['year'] = today.year
        self.month = self.kwargs['month'] = today.month
        self.day = self.kwargs['day'] = today.day
        return today
    
    def get_datebook(self, filters):
        try:
            obj = Datebook.objects.get(author__username=self.kwargs['author'], **filters)
        except Datebook.DoesNotExist:
            author = get_object_or_404(User, username=self.kwargs['author'])
            obj = Datebook(author=author, period=datetime.date(self.year, self.month, self.day))
            obj.save()
        return obj


class OwnerOrPermissionRequiredMixin(PermissionRequiredMixin):
    """
    Act like 'PermissionRequiredMixin' but pass permission test if the user is 
    the object owner.
    
    Object owner must be in a class attribute named "author". With 'AuthorKwargsMixin' 
    usage, you simply have to put just after like this :
    
        class MyView(AuthorKwargsMixin, OwnerOrPermissionRequiredMixin, ....):
            pass
    
    Then you don't have to do anything for the 'author' attribute. This works for any 
    other class that inherit from 'AuthorKwargsMixin'.
    """
    def dispatch(self, request, *args, **kwargs):
        if getattr(self, 'author', None) is None:
            raise ImproperlyConfigured(
                "'OwnerOrPermissionRequiredMixin' requires "
                "'author' attribute to be set.")
        
        # If the user is the object owner pass permission tests 
        # using 'PermissionRequiredMixin' ancestor
        if request.user.is_authenticated() and request.user == self.author:
            return super(PermissionRequiredMixin, self).dispatch(
                request, *args, **kwargs)
        
        # If user is not the Object owner, continue to check for permission
        return super(OwnerOrPermissionRequiredMixin, self).dispatch(
            request, *args, **kwargs)



# -*- coding: utf-8 -*-
"""
Forms for year forms
"""
import datetime

from django import forms
from django.utils.translation import ugettext as _
from django.contrib.auth.models import User

from crispy_forms.helper import FormHelper
from crispy_forms_foundation.layout import Submit

from datebook.models import Datebook
from datebook.forms import CrispyFormMixin

class DatebookYearForm(CrispyFormMixin, forms.Form):
    """
    Datebook year form
    """
    crispy_form_helper_path = 'datebook.forms.crispies.year_helper'
    
    def __init__(self, author, queryset, *args, **kwargs):
        self.author = author
        
        super(DatebookYearForm, self).__init__(*args, **kwargs)
        super(forms.Form, self).__init__(*args, **kwargs)
        
        self.today = datetime.date.today()
        existing_years = [item.year for item in queryset]
        available_years = [(k, k) for k in range(self.today.year-3, self.today.year+4) if k not in existing_years]
        
        self.fields['year'] = forms.ChoiceField(label=_('year'), choices=available_years)
    
    def save(self, *args, **kwargs):
        year = int(self.cleaned_data['year'])
        # Default opened datebook is for the first month of the given year
        period = [year, 1, 1]
        # Open the datebook for the current month if the given year is the current year
        if year == self.today.year:
            period = [year, self.today.month, 1]
            
        instance = Datebook(
            author=self.author,
            period=datetime.date(*period)
        )
        instance.save()
        
        return instance



# -*- coding: utf-8 -*-
"""
Root url's map for application
"""
from django.conf.urls import *

from datebook.views import IndexView
from datebook.views.author import DatebookAuthorView
from datebook.views.year import DatebookYearView
from datebook.views.month import DatebookMonthView, DatebookMonthAddView, DatebookMonthFormView
from datebook.views.day import DayEntryFormCreateView, DayEntryDetailView, DayEntryFormEditView, DayEntryCurrentView

urlpatterns = patterns('',
    url(r'^$', IndexView.as_view(), name='datebook-index'),
    
    url(r'^create/$', DatebookMonthFormView.as_view(), name='datebook-create'),
    
    url(r'^(?P<author>\w+)/$', DatebookAuthorView.as_view(), name='datebook-author'),
    
    url(r'^(?P<author>\w+)/current-day/$', DayEntryCurrentView.as_view(), name='datebook-current-day'),
    
    url(r'^(?P<author>\w+)/(?P<year>\d{4})/$', DatebookYearView.as_view(), name='datebook-author-year'),
    url(r'^(?P<author>\w+)/(?P<year>\d{4})/add/(?P<month>\d{1,2})/$', DatebookMonthAddView.as_view(), name='datebook-author-month-add'),
    
    url(r'^(?P<author>\w+)/(?P<year>\d{4})/(?P<month>\d{1,2})/$', DatebookMonthView.as_view(), name='datebook-author-month'),
    
    url(r'^(?P<author>\w+)/(?P<year>\d{4})/(?P<month>\d{1,2})/add/(?P<day>\d{1,2})/$', DayEntryFormCreateView.as_view(), name='datebook-author-day-add'),
    
    url(r'^(?P<author>\w+)/(?P<year>\d{4})/(?P<month>\d{1,2})/(?P<day>\d{1,2})/$', DayEntryDetailView.as_view(), name='datebook-author-day'),
    url(r'^(?P<author>\w+)/(?P<year>\d{4})/(?P<month>\d{1,2})/(?P<day>\d{1,2})/edit/$', DayEntryFormEditView.as_view(), name='datebook-author-day-edit'),
)



# -*- coding: utf-8 -*-
"""
Datebook author views
"""
import datetime

from django.views import generic
from django.shortcuts import get_object_or_404
from django.core.urlresolvers import reverse
from django.contrib.auth.models import User

from braces.views import LoginRequiredMixin

from datebook.models import Datebook
from datebook.forms.year import DatebookYearForm
from datebook.utils.views import ListAppendView

class DatebookAuthorView(LoginRequiredMixin, ListAppendView):
    """
    User datebook index
    
    Display all years that have one or more Datebooks for the given user
    """
    model = Datebook
    form_class = DatebookYearForm
    paginate_by = None
    locked_form = False
    template_name = "datebook/author_index.html"
    
    def get_queryset(self, *args, **kwargs):
        self.author = get_object_or_404(User, username=self.kwargs['author'])
        self.queryset = self.model.objects.filter(author=self.author).dates('period', 'year')
        return super(DatebookAuthorView, self).get_queryset(*args, **kwargs)

    def get_form(self, form_class):
        if not self.request.user.is_superuser and self.request.user != self.author and not self.request.user.has_perm('datebook.add_datebook'):
            return None
        return super(DatebookAuthorView, self).get_form(form_class)
        
    def get_form_kwargs(self, **kwargs):
        kwargs = super(DatebookAuthorView, self).get_form_kwargs(**kwargs)
        kwargs.update({
            'author': self.author,
            'queryset': self.queryset,
        })
        return kwargs
        
    def format_year_list(self, queryset):
        """
        Reformate the paginated queryset to replace items with a tuple containing the 
        item's year, datetime and it's opened month counter
        """
        return [(item.year, item, self.count_opened_months(item.year)) for item in queryset]
        
    def count_opened_months(self, year):
        return Datebook.objects.filter(author=self.author, period__year=year).count()
        
    def get_context_data(self, **kwargs):
        context = super(DatebookAuthorView, self).get_context_data(**kwargs)
        context.update({
            'author': self.author,
            'today': datetime.datetime.today(),
            'object_list': self.format_year_list(context['object_list']),
        })
        return context

    def get_success_url(self):
        return reverse('datebook-author-year', kwargs={
            'author': self.author,
            'year': self.object.period.year,
        })



# -*- coding: utf-8 -*-
"""
Forms for day forms
"""
from django import forms
from django.utils.translation import ugettext as _

from crispy_forms.helper import FormHelper
from crispy_forms_foundation.layout import Layout, Fieldset, SplitDateTimeField, RowFluid, Column, ButtonHolder, Submit

from datebook.models import DayEntry
from datebook.forms import CrispyFormMixin

DATETIME_FORMATS = {
    'input_date_formats': ['%d/%m/%Y'],
    'input_time_formats': ['%H:%M'], 
    'widget': forms.SplitDateTimeWidget(date_format='%d/%m/%Y', time_format='%H:%M'),
}

class DayEntryForm(CrispyFormMixin, forms.ModelForm):
    """
    DayEntry form
    """
    crispy_form_helper_path = 'datebook.forms.crispies.day_helper'
    
    start_datetime = forms.SplitDateTimeField(label=_('start'), **DATETIME_FORMATS)
    stop_datetime = forms.SplitDateTimeField(label=_('stop'), **DATETIME_FORMATS)

    def __init__(self, datebook, day, *args, **kwargs):
        self.datebook = datebook
        self.daydate = datebook.period.replace(day=day)
        
        self.crispy_form_helper_kwargs = {
            'has_next': False,
            'form_action': kwargs.pop('form_action'),
        }
        
        # Pass initial data for start and stop to their clone with SplitDateTimeField
        if 'start' in kwargs['initial']:
            kwargs['initial']['start_datetime'] = kwargs['initial']['start']
        if 'stop' in kwargs['initial']:
            kwargs['initial']['stop_datetime'] = kwargs['initial']['stop']
        # For existing instance (in edit mode) pass the start and stop values to their 
        # clone with SplitDateTimeField via initial datas
        if kwargs.get('instance'):
            kwargs['initial']['start_datetime'] = kwargs['instance'].start
            kwargs['initial']['stop_datetime'] = kwargs['instance'].stop
                
        
        super(DayEntryForm, self).__init__(*args, **kwargs)
        super(forms.ModelForm, self).__init__(*args, **kwargs)
    
    def clean_start_datetime(self):
        start = self.cleaned_data['start_datetime']
        # Day entry can't start before the targeted day date
        if start and start.date() < self.daydate:
            raise forms.ValidationError(_("You can't start a day before itself"))
        # Day entry can't start after the targeted day date
        if start and start.date() > self.daydate:
            raise forms.ValidationError(_("You can't start a day after itself"))
        
        return start
    
    def clean_stop_datetime(self):
        start = self.cleaned_data.get('start_datetime')
        stop = self.cleaned_data['stop_datetime']
        # Day entry can't stop before the start
        if start and stop and stop <= start:
            raise forms.ValidationError(_("Stop time can't be less or equal to start time"))
        # Day entry can't stop more than one day to the targeted day date
        if stop and stop.date() > self.daydate.replace(day=self.daydate.day+1):
            raise forms.ValidationError(_("Stop time can't be more than the next day"))
        
        return stop
    
    # TODO: overtime must not be more than effective worked time
    #def clean_overtime(self):
        #overtime = self.cleaned_data.get('overtime')
        #return overtime
    
    # TODO
    #def clean_pause(self):
        #start = self.cleaned_data.get('start_datetime')
        #stop = self.cleaned_data.get('stop_datetime')
        #pause = self.cleaned_data['pause']
        ## Pause time can't be more than elapsed time between start and stop
        #if start and stop and pause and False:
            #raise forms.ValidationError("Pause time is more than the elapsed time")
        
        #return pause
    
    def save(self, *args, **kwargs):
        instance = super(DayEntryForm, self).save(commit=False, *args, **kwargs)
        instance.start = self.cleaned_data['start_datetime']
        instance.stop = self.cleaned_data['stop_datetime']
        instance.datebook = self.datebook
        instance.activity_date = self.daydate
        instance.save()
        
        return instance
    
    class Meta:
        model = DayEntry
        exclude = ('datebook', 'activity_date', 'start', 'stop')
        widgets = {
            'pause': forms.TimeInput(format=DATETIME_FORMATS['input_time_formats'][0]),
            'overtime': forms.TimeInput(format=DATETIME_FORMATS['input_time_formats'][0]),
        }


# -*- coding: utf-8 -*-
"""
Common views
"""
from django.views import generic
from django.shortcuts import get_object_or_404
from django.contrib.auth.models import User

from braces.views import LoginRequiredMixin

from datebook.models import Datebook

class IndexView(LoginRequiredMixin, generic.TemplateView):
    """
    Index view
    
    Display all user that have one or more Datebooks
    """
    template_name = "datebook/index.html"
    
    def get(self, request, *args, **kwargs):
        
        context = {
            'object_list': Datebook.objects.filter(author__is_active=True).values('author__username').distinct(),
        }
        
        return self.render_to_response(context)



# -*- coding: utf-8 -*-
"""
Day entry views
"""
import datetime

from django.conf import settings
from django.views import generic
from django.core.urlresolvers import reverse
from django.contrib.auth.models import User
from django.shortcuts import get_object_or_404

from braces.views import LoginRequiredMixin

from datebook.models import Datebook, DayEntry
from datebook.mixins import AuthorKwargsMixin, DatebookCalendarMixin, DatebookCalendarAutoCreateMixin, OwnerOrPermissionRequiredMixin
from datebook.forms.day import DayEntryForm
from datebook.utils import week_from_date

class DayEntryBaseFormView(DatebookCalendarMixin, OwnerOrPermissionRequiredMixin):
    """
    DayEntry base form view
    """
    model = DayEntry
    context_object_name = "dayentry"
    template_name = "datebook/day/form.html"
    ajax_template_name = "datebook/day/form_fragment.html"
    form_class = DayEntryForm
    permission_required = 'datebook.add_dayentry'
    raise_exception = True

    def get_form(self, form_class):
        """
        Add required args to form instance
        """
        return form_class(self.datebook, self.day, **self.get_form_kwargs())
    
    def switch_template(self, request):
        if request.is_ajax():
            return self.ajax_template_name
        return self.template_name
    
    def get(self, request, *args, **kwargs):
        self.template_name = self.switch_template(request)
        self.datebook = self.get_datebook({'period__year': self.year, 'period__month': self.month})
        
        return super(DayEntryBaseFormView, self).get(request, *args, **kwargs)
    
    def post(self, request, *args, **kwargs):
        self.template_name = self.switch_template(request)
        self.datebook = self.get_datebook({'period__year': self.year, 'period__month': self.month})
        
        return super(DayEntryBaseFormView, self).post(request, *args, **kwargs)
    
    def get_form_kwargs(self):
        """
        Add post form url
        """
        kwargs = super(DayEntryBaseFormView, self).get_form_kwargs()
        kwargs.update({'form_action': '.'})
        return kwargs

    def get_success_url(self):
        return reverse('datebook-author-month', kwargs={
            'author': self.author,
            'year': self.object.activity_date.year,
            'month': self.object.activity_date.month,
        })
        
    def get_next(self):
        return None


class DayEntryFormCreateView(DayEntryBaseFormView, generic.CreateView):
    """
    DayEntry form create view
    """
    def get_form_kwargs(self):
        """
        Put the initial data for the datetime start and stop
        
        NOTE: Time start and stop should comes from the User profile settings (date 
              allways come from the datebook period and the required day).
        """
        kwargs = super(DayEntryFormCreateView, self).get_form_kwargs()
        day_date = self.datebook.period.replace(day=self.day)
        time_start = datetime.time(9, 0)
        time_stop = datetime.time(18, 0)
        kwargs.update({
            'initial': {
                'start' : datetime.datetime.combine(day_date, time_start),
                'stop' : datetime.datetime.combine(day_date, time_stop),
                'pause' : datetime.time(1, 0),
            },
            'form_action': reverse('datebook-author-day-add', kwargs={
                'author': self.author,
                'year': self.year,
                'month': self.month,
                'day': self.day,
            })
        })
        return kwargs


class DayEntryFormEditView(DayEntryBaseFormView, generic.UpdateView):
    """
    DayEntry form edit view
    """
    permission_required = 'datebook.change_dayentry'
    
    def get_object(self):
        """
        Add required args to form instance
        """
        return self.datebook.dayentry_set.get(activity_date__day=self.day)
    
    def get_form_kwargs(self):
        """
        Add post form url
        """
        kwargs = super(DayEntryBaseFormView, self).get_form_kwargs()
        kwargs.update({
            'form_action': reverse('datebook-author-day-edit', kwargs={
                'author': self.author,
                'year': self.year,
                'month': self.month,
                'day': self.day,
            })
        })
        return kwargs


class DayEntryCurrentView(AuthorKwargsMixin, OwnerOrPermissionRequiredMixin, generic.RedirectView):
    """
    Redirect to the form for the current day. If entry allready exists this will 
    redirect to edit form, else to add form.
    
    If the month datebook does not exist for the current day, this will create it 
    before redirect.
    """
    permission_required = 'datebook.add_dayentry'
    raise_exception = True
    permanent = False
    
    def get_current_date(self):
        """
        Get the current date
        """
        today = datetime.date.today()
        self.year = self.kwargs['year'] = today.year
        self.month = self.kwargs['month'] = today.month
        self.day = self.kwargs['day'] = today.day
        return today
    
    def get_datebook(self):
        """
        Get or Create datebook
        """
        try:
            obj = Datebook.objects.get(author__username=self.kwargs['author'], period__year=self.year, period__month=self.month)
        except Datebook.DoesNotExist:
            self.author = get_object_or_404(User, username=self.kwargs['author'])
            obj = Datebook(author=self.author, period=datetime.date(self.year, self.month, 1))
            obj.save()
        else:
            self.author = obj.author
        return obj
    
    def get_redirect_url(self, author):
        """
        Get Create or Edit dayentry url
        """
        today = self.get_current_date()
        datebook = self.get_datebook()
        
        try:
            obj = DayEntry.objects.get(datebook=datebook, activity_date=datetime.date(self.year, self.month, self.day))
        except DayEntry.DoesNotExist:
            return reverse('datebook-author-day-add', kwargs={
                'author': self.author,
                'year': self.year,
                'month': self.month,
                'day': self.day,
            })
        else:
            return reverse('datebook-author-day-edit', kwargs={
                'author': self.author,
                'year': self.year,
                'month': self.month,
                'day': self.day,
            })


class DayEntryDetailView(LoginRequiredMixin, DatebookCalendarMixin, generic.TemplateView):
    """
    DayEntry detail view
    """
    model = DayEntry
    template_name = "datebook/day/detail_fragment.html"
        
    def get_previous(self):
        try:
            obj = self.object.get_previous_by_activity_date(**{
                'datebook__author': self.author,
                'activity_date__month': self.object.activity_date.month,
                'activity_date__year': self.object.activity_date.year,
            })
            print "FOO", obj
        except DayEntry.DoesNotExist:
            pass
        else:
            return obj
        return None
        
    def get_next(self):
        try:
            obj = self.object.get_next_by_activity_date(**{
                'datebook__author': self.author,
                'activity_date__month': self.object.activity_date.month,
                'activity_date__year': self.object.activity_date.year,
            })
        except DayEntry.DoesNotExist:
            pass
        else:
            return obj
        return None
        
    def get_object(self, **kwargs):
        return get_object_or_404(DayEntry, datebook=self.datebook, activity_date=datetime.date(self.year, self.month, self.day))
        #try:
            #obj = DayEntry.objects.get(datebook=self.datebook, activity_date=datetime.date(self.year, self.month, self.day))
        #except DayEntry.DoesNotExist:
            #pass
        #else:
            #return obj
        #return None
    
    def get_context_data(self, **kwargs):
        context = super(DayEntryDetailView, self).get_context_data(**kwargs)
        context.update({
            'datebook': self.datebook,
            'object': self.object,
            'previous_day': self.previous_day,
            'next_day': self.next_day,
        })
        return context
    
    def get(self, request, *args, **kwargs):
        self.datebook = self.get_datebook({'period__year': self.year, 'period__month': self.month})
        self.object = self.get_object()
        self.previous_day = self.get_previous()
        self.next_day = self.get_next()
        
        context = self.get_context_data(**kwargs)
        
        return self.render_to_response(context)

